/*
	Copyright (C) 2002 Slurdge (aka Aurlien Pocheville).
	
	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 *	Petit programme qui convertit les relevs Web de la poste en .qif
 *
 *	\author Aurlien Pocheville
 *	\since 2002-01-03
 *
 */

//Includes
#include <cstdio>
#include <vector>
#ifdef _WIN32
#include <windows.h>
#endif

#define NUM_START_LINES		8		///< Nombre de lignes qu'il faut sauter au dbut.
#define START_BUFFER_SIZE	512		///< Taille du buffer des lignes qu'il faut sauter.
#define GPL_LICENCE "Copyright (C) 2002 Slurdge.\n\
\n\
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program; if not, write to the Free Software\n\
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n"


/**
 *	Function that prints out
 *	the program normal use.
 */
void printf_usage(char * _pcProgName)
{
	printf("Usage : %s <source_file> [destination_file]\n",_pcProgName);
	printf("Convert the La Poste internet file to a Money .qif file\n");
	printf("source_file : The source file. Must be a La Poste .tsv file\n");
	printf("destination_file : The destination file. Must be a .qif file.\n");
	printf("If destination_file is ommited, a file with a .qif extension and the same ");
	printf("name as the source file is created.\n");
}

/**
 *	Struct that holds
 *	a bank operation
 */
struct sOperation
{
	std::string		m_strName;
	std::string		m_strDate;
	std::string		m_strValue;
	
};

/**
 *	This function strips spaces in a
 *	string. This is useful when
 *	La Poste puts additionnals
 *	spaces in its numbers (eg 1 345,99)
 */
char * strip_spaces(const char * _pcInString)
{
	char * pcOutString = new char[strlen(_pcInString)+1];

	for (unsigned int i = 0, j = 0; i < strlen(_pcInString) + 1; )
	{
		if (_pcInString[i] != ' ') //' ' = 32 = space
		{
			pcOutString[j++] = _pcInString[i++];
		}
		else
		{
			i++;
		}
	}

	return pcOutString;
}

/**
 *	Main Function
 *	It first does a parsing, and
 *	then produces output.
 */
void main(int _iArgCounter, char * _pstrArgValues[])
{

	//Variables
	FILE	*	pfSourceFile = NULL;
	FILE	*	pfDestFile = NULL;
	char		strBuffer[START_BUFFER_SIZE];
	char		strBufferv2[START_BUFFER_SIZE];
	std::vector<sOperation>	vOperations;

	printf("tsv2qif (C) Slurdge 2002-2007\n");
	printf("-----------------------------\n");

	if ( (_iArgCounter != 2) && (_iArgCounter != 3) )
	{
		printf_usage(_pstrArgValues[0]);
		printf(GPL_LICENCE);
		//delete strBuffer;
		exit(-1);
	}

	printf("Opening source file ...\n");
	if ( (pfSourceFile = fopen(_pstrArgValues[1],"rt")) == NULL )
	{
		printf("Error : Cannot open source file.\n");
		printf_usage(_pstrArgValues[0]);
		//delete strBuffer;
		exit(-2);
	}

	printf("Parsing source file ...\n");

	//On enlve les lignes du dbut
	for (int i = 0; i < NUM_START_LINES ; i++)
	{
		if (fgets(strBuffer,START_BUFFER_SIZE,pfSourceFile) == NULL)
		{
			printf("Error : Check the validity of your tsv file\n");
			printf_usage(_pstrArgValues[0]);
			//delete strBuffer;
			exit(-3);
		}
#ifdef _DEBUG
		else
		{
			printf("%s",strBuffer);
		}
#endif
	}
	//delete strBuffer;

	//On construit la structure
	int iNumOperations = 0;
	char *strFirstDate;
	char *strLastDate;
	bool bFirstTime = true;
	while (!feof(pfSourceFile))
	{
		char	*	strDate = new char[32];
		char	*	strName = new char[128];
		char	*	strEuroValue = new char[16];
		char	*	strFrancValue = new char[16];
		sOperation	newOperation;

		if (fgets(strBuffer,START_BUFFER_SIZE,pfSourceFile) == NULL)
			break;

		bool		bVersion2 = false;
		
		if (strchr(strBuffer,'"') != 0)
			bVersion2 = true;
		
		if (bVersion2)
		{
			if (strchr(strchr(strBuffer,'"')+1,'"') == 0)
			{
				if (fgets(strBufferv2,START_BUFFER_SIZE,pfSourceFile) == NULL)
					break;
				strBuffer[strlen(strBuffer)-1] = ' ';
				strcpy(&(strBuffer[strlen(strBuffer)]),strBufferv2);
			}
			//strcpy(strBuffer,strBuffer+1);
			//strBuffer[strlen(strBuffer)-2]='\0';
		}

		if (feof(pfSourceFile))
			break;

		char	*	pcTabPos;
		char	*	strTempBuffer = strBuffer;

		if ( (pcTabPos = strchr(strTempBuffer,'\t')) == NULL )
		{
			printf("Error : Check the validity of your tsv file\n");
			printf_usage(_pstrArgValues[0]);
			//delete strBuffer;
			exit(-3);
		}
		else
		{
			strncpy(strDate,strTempBuffer,pcTabPos - strTempBuffer + 1);
			strDate[pcTabPos - strTempBuffer + 1] = 0;
			strTempBuffer = pcTabPos + 1;
		}

		if ( (pcTabPos = strchr(strTempBuffer,'\t')) == NULL )
		{
			printf("Error : Check the validity of your tsv file\n");
			printf_usage(_pstrArgValues[0]);
			//delete strBuffer;
			exit(-3);
		}
		else
		{
			if (!bVersion2)
			{
				strncpy(strName,strTempBuffer,pcTabPos - strTempBuffer + 1);
				strName[pcTabPos - strTempBuffer + 1] = 0;
			}
			else
			{
				strncpy(strName,strTempBuffer+1,pcTabPos - strTempBuffer - 2);
				strName[pcTabPos - strTempBuffer - 2] = 0;
			}
			strTempBuffer = pcTabPos + 1;
		}

		if ( (pcTabPos = strchr(strTempBuffer,'\t')) == NULL )
		{
			printf("Error : Check the validity of your tsv file\n");
			printf_usage(_pstrArgValues[0]);
			//delete strBuffer;
			exit(-3);
		}
		else
		{
			strncpy(strEuroValue,strTempBuffer,pcTabPos - strTempBuffer + 1);
			strEuroValue[pcTabPos - strTempBuffer + 1] = 0;
			strTempBuffer = pcTabPos + 1;
		}

		if ( (pcTabPos = strchr(strBuffer,'\n')) == NULL )
		{
			printf("Error : Check the validity of your tsv file\n");
			printf_usage(_pstrArgValues[0]);
			//delete strBuffer;
			exit(-3);
		}
		else
		{
			strncpy(strFrancValue,strTempBuffer,pcTabPos - strTempBuffer + 1);
			strFrancValue[pcTabPos - strTempBuffer + 1] = 0;
			strTempBuffer = pcTabPos + 1;
		}
		iNumOperations++;
		if (bFirstTime)
		{
			strFirstDate = strDate;
			bFirstTime = false;
		}
		else
		{
			strLastDate = strDate;
		}
		newOperation.m_strDate = std::string(strDate);
		newOperation.m_strName = std::string(strName);
		newOperation.m_strValue = std::string(strip_spaces(strEuroValue));
		//Si jamais la valeur comporte une virgule , la remplacer par un point
		std::string::size_type stPos = 0;
		while ( ( stPos = newOperation.m_strValue.find( "," , stPos ) ) != std::string::npos )
		{
			newOperation.m_strValue.replace( stPos, 1 , "." );
			stPos ++; // new starting point of search is just after ','
		}

		vOperations.push_back(newOperation);

		printf("Entry detected : \n\tDate:%s\n\tName:%s\n\tValue:%s (Euros)\n",newOperation.m_strDate.c_str(),newOperation.m_strName.c_str(),newOperation.m_strValue.c_str());
	}

	printf("Parsing complete\n");

	std::string	strOutputFilename;
	if (_iArgCounter == 2)
	{
		strOutputFilename = _pstrArgValues[1];
		strOutputFilename.resize(strOutputFilename.size() - 4);
		strOutputFilename += ".qif";
	}
	else
	{
		strOutputFilename = _pstrArgValues[2];
	}

	printf("Opening destination file ...\n");
	if ( (pfDestFile = fopen(strOutputFilename.c_str(),"wt")) == NULL )
	{
		printf("Error : Cannot open destination file.\n");
		printf_usage(_pstrArgValues[0]);
		//delete strBuffer;
		exit(-2);
	}

	printf("Populating destination file ...\n");
	fprintf(pfDestFile,"!Type:Bank\n");
	for (std::vector<sOperation>::iterator It = vOperations.begin(); It != vOperations.end() ; ++It)
	{
		fprintf(pfDestFile,"D%s\n",(*It).m_strDate.c_str());
		fprintf(pfDestFile,"T%s\n",(*It).m_strValue.c_str());
		fprintf(pfDestFile,"P%s\n",(*It).m_strName.c_str());
		fprintf(pfDestFile,"^\n");
	}
	
	printf("Conversion is Done.");

	char	strResult[1024];
#ifdef _WIN32
	sprintf(strResult,"Rsum:\r\nNombre d'oprations:%i\r\nDernire opration:%s\r\nPremire opration:%s",iNumOperations,strFirstDate,strLastDate);
	MessageBox(NULL,strResult,"Conversion effectue",MB_OK | MB_ICONINFORMATION);
#else
	printf("Rsum:\nNombre d'oprations:%i\nDernire opration:%s\nPremire opration:%s\n",iNumOperations,strFirstDate,strLastDate);
#endif

	//delete strBuffer;
	fclose(pfSourceFile);
	fclose(pfDestFile);
	exit(0);
}